::mercModelPath <- "models/player/mercenary.mdl";

PrecacheModel(mercModelPath);
PrecacheSound("respawn.wav")

PrecacheSound("vo/mercenary_paincrticialdeath01.mp3")
PrecacheSound("vo/mercenary_paincrticialdeath02.mp3")
PrecacheSound("vo/mercenary_paincrticialdeath03.mp3")
PrecacheSound("vo/mercenary_paincrticialdeath04.mp3")

vo_death_sounds <- ["vo/mercenary_paincrticialdeath01.mp3", "vo/mercenary_paincrticialdeath02.mp3", "vo/mercenary_paincrticialdeath03.mp3", "vo/mercenary_paincrticialdeath04.mp3"]

OnSelfGameEvent("player_spawn", 0, function(player)
{
    self.SetScriptOverlayMaterial("");
    //self.AddHudHideFlags(/*HIDEHUD_MISCSTATUS |*/ HIDEHUD_METAL);

    if (redTeamOnly && self.GetTeam() != TF_TEAM_RED)
    {
        self.SwitchTeam(TF_TEAM_RED);
        self.ForceRespawn();
    }
});

OnSelfGameEvent("player_spawn_post", 0, function(player)
{
    //Delete weapons and cosmetics
    local itemsToKill = [];
    for (local item = self.FirstMoveChild(); item != null; item = item.NextMovePeer())
        if (startswith(item.GetClassname(), "tf_we"))
            itemsToKill.push(item);
    foreach (item in itemsToKill)
        item.Kill();

    SetPropInt(self, "m_Shared.m_bShieldEquipped", 0);
    //Equip new weapons
    EquipWeapon(GetWeaponTypeByName("crowbar"));
    self.Weapon_Switch(EquipWeapon(GetCurrentGunGameWeapon(self)));

    //Refresh model, movespeed and HP
    RefreshCharacter(GetCurrentGunGameWeapon(self), false);

    //Add Merc's attributes
    self.AddCustomAttribute("cancel falling damage", 1, -1);
    self.AddCustomAttribute("crit mod disabled hidden", 0, -1);
    self.AddCustomAttribute("voice pitch scale", 0, -1);
    self.AddCustomAttribute("no double jump", 1, -1);

    //Steam-bound clothing color
    SetEntityColor(CalcMercColor());

    //On-spawn invul
    SetPropInt(self, "m_bForcedSkin", 1);
    SetPropInt(self, "m_nForcedSkin", 5);
    if (GetRoundState() == 3)
        self.AddCustomAttribute("no_attack", 1, 5.2);
    self.AddCondEx(TF_COND_INVULNERABLE, GetRoundState() == 3 ? 5 : 3, null);
    RunWithDelay( GetRoundState() == 3 ? 5 : 3, function()
    {
        SetPropInt(self, "m_nForcedSkin", 4);
    }, self)

    //Spawn sound
    if (GetRoundState() >= 4)
        EmitSoundEx({
            sound_name = "respawn.wav",
            volume = 1,
            sound_level = 150,
            flags = 1,
            channel = 0,
            entity = self,
            speaker_entity = self
        });
});

function RefreshCharacter(weaponTT, keepCurrentHP = true)
{
    local playerClass = "playerclass" in weaponTT ? weaponTT.playerclass : self.GetPlayerClass();
    local oldHP = self.GetHealth();
    self.SetPlayerClass(playerClass);
    self.AddCustomAttribute("move speed bonus",
        1.04 / MERC_MOVESPEED_MULTIPLIER[playerClass],
        -1);

    self.SetCustomModelWithClassAnimations(mercModelPath);

    self.SetHealth(keepCurrentHP ? oldHP : 150);
    self.SetMaxHealth(150);
    self.RemoveCustomAttribute("max health additive penalty");
    self.AddCustomAttribute("max health additive penalty", MERC_HP_OFFSETS[playerClass], -1);

    RefreshModels();
}

OnSelfGameEvent("player_death", 0, function()
{
    EmitSoundEx({
        sound_name = vo_death_sounds[RandomInt(0, vo_death_sounds.len() - 1)],
        entity = self,
        sound_level = 90,
    });
    RunWithDelay(3, function()
    {
        self.ForceRespawn();
    });
});

OnSelfGameEvent("player_death_post", 0, function()
{
    local ragdoll = GetPropEntity(self, "m_hRagdoll");
    SetPropBool(ragdoll, "m_bGib", false);
    SetPropInt(ragdoll, "m_clrRender", 4444444);
});

AddTimer(-1, 0, function()
{
    if (GetPropInt(self, "m_nButtons") & IN_ATTACK && GetRoundState() > 3)
    {
        self.RemoveCond(TF_COND_INVULNERABLE);
        SetPropInt(self, "m_nForcedSkin", 4);
    }
})

function SetEntityColor(rgb)
{
    local color = rgb[0] | (rgb[1] << 8) | (rgb[2] << 16);
    SetPropInt(self, "m_clrRender", color);
}

function CalcMercColor()
{
    local steamId = GetPropString(self, "m_szNetworkIDString");
    local indexStr = steamId.len() > 5 ? steamId.slice(5) : "4444444";
    local index = indexStr.tointeger();
    return [index % 256, (index >> 8) % 256, (index >> 16) % 256, 255];
}